package com.example.test;

import org.apache.commons.io.FileUtils;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.event.process.DefaultProcessEventListener;
import org.drools.event.process.ProcessCompletedEvent;
import org.drools.event.process.ProcessNodeLeftEvent;
import org.drools.event.process.ProcessNodeTriggeredEvent;
import org.drools.event.process.ProcessStartedEvent;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;

public class FlowTest {

    private StatefulKnowledgeSession ksession;

    public FlowTest() {

    }

    class EventListener extends DefaultProcessEventListener {

        @Override
        public void beforeNodeLeft(ProcessNodeLeftEvent event) {
        }

        @Override
        public void beforeNodeTriggered(ProcessNodeTriggeredEvent event) {
            long nodeId = event.getNodeInstance().getNodeId();
            String name = event.getNodeInstance().getNodeName();
            System.err.println("@node: " + nodeId + " - " + name);
        }

        @Override
        public void beforeProcessCompleted(ProcessCompletedEvent event) {
        }

        @Override
        public void beforeProcessStarted(ProcessStartedEvent event) {
        }

        @Override
        public void afterNodeLeft(ProcessNodeLeftEvent event) {
        }

        @Override
        public void afterNodeTriggered(ProcessNodeTriggeredEvent event) {
        }

        @Override
        public void afterProcessCompleted(ProcessCompletedEvent event) {
            System.err.println("Process completed");
        }

        @Override
        public void afterProcessStarted(ProcessStartedEvent event) {
            System.err.println("Process started");
        }
    }

    public void run() throws Exception {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();

        String ruleString = FileUtils.readFileToString(FileUtils.toFile(this.getClass().getResource("/resources/rules.drl")));
        String flowString = FileUtils.readFileToString(FileUtils.toFile(this.getClass().getResource("/resources/flow.bpmn2")));

        kbuilder.add(ResourceFactory.newByteArrayResource(flowString.getBytes()),
                ResourceType.BPMN2);
        kbuilder.add(ResourceFactory.newByteArrayResource(ruleString.getBytes()),
                ResourceType.DRL);
        final KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();

        if (kbuilder.hasErrors()) {
            System.err.println(kbuilder.getErrors().toString());
        }

        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
        ksession = kbase.newStatefulKnowledgeSession();
        ksession.insert(new Document());
        ksession.addEventListener(new EventListener());
        new Thread(new Runnable() {
            @Override
            public void run() {
                ksession.startProcess("TestFlow");
                ksession.fireAllRules();
            }
        }).run();
    }

    public void insertEvent(Event fact) throws Exception {
        ksession.insert(fact);
        ksession.fireAllRules();
    }

    public static void main(String args[]) throws Exception {
        FlowTest pt = new FlowTest();
        pt.run();
        Thread.sleep(2000);
        System.err.println("Inserting create event");
        pt.insertEvent(new Event("Created"));
        Thread.sleep(3000);
    }
}
